% ICI (Intersection of Confidence Intervals) algorithm for adaptive selection of image estimates
% -------------------------------------------------------------------------------------------------
%
% Complete implementation of the ICI, including pointwise-adaptive variance
% (AV) and optional embedded weighted order-statistics filtering (WOS).
%
% [YY, IN,std_opt]=function_ICI(B,std_B,gam,[direction|W],O);
%
% INPUTS:
% B         : an array of image estimates obtained for set of increasing window sizes, size(B)=N1*N2*lenh
% std_B     : an array with the corresponding estimates' standard deviations
% gam       : the ICI Gamma threshold (a scalar)
%
% OPTIONAL INPUTS:
% direction : direction of the convolutional LPA kernels used for
%             estimation; when specified it enables the directional WOS
%             filters within ICI.
% W         : the cell array providing the weight matrices for the weighted
%             order-statistics (WOS) filtering for each ICI iteration.
% O         : vector of the weighted statistics offsets.
%              ( offset=0 corresponds to standard weighted median
%                e.g. W={ones(3),ones(3),ones(1)} and O=(0 0 0)
%                     will result in 2 recursive 3x3 median filters )
%              weights have to be odd1 x odd2 matrices and sum of weights + offset
%              should be odd
%
%
% OUTPUTS:
% YY      : is a N1xN2 matrix of the adaptive estimates
% IN      : is a N1xN2 matrix of the adaptive estimate/scale indexes (1 corresponds to the smallest scale)
% std_opt : is a N1xN2 matrix of of the adaptive estimates' standard deviations
%
%
%function_ICI_1D.m
% ---------------------------------------------------------------
% B=[0.1 1 2];
% std_B=[0.1 1 2]/10
% gam=1
function [YY, IN, std_opt]=function_ICI_1D(B,std_B,gam);
[lenh]=length(B);
D=gam*std_B;
U=B+D;
L=B-D;
YY=B(1);
std_opt=std_B(1);
TT(1)=1;
IN=1;
for s=2:lenh
    L(s)=max(L(s-1),L(s));         % updates L and U
    U(s)=min(U(s-1),U(s));
    TT(s)=(U(s)>=L(s));                  %checks for non-empty intersection
    TT(s)=TT(s)&TT(s-1);
    YY=TT(s)*B(s)+~TT(s)*YY;  % updates estimates
    IN=TT(s)*s+~TT(s)*IN;  % updates indexes
    std_opt=TT(s)*std_B(s)+~TT(s)*std_opt; 
end
%keyboard

    

